<?php

use QBNK\QBank\API\Model\MediaUsage;

/**
 * @file
 * Functions related to the tracking the file usage of qbank media.
 */

function qbank_dam_entity_insert($entity, $entity_type) {
  _qbank_filter_add_file_usage_from_fields($entity_type, $entity);
  _qbank_add_usage($entity);
}

function qbank_dam_entity_update($entity, $entity_type) {
  _qbank_filter_add_file_usage_from_fields($entity_type, $entity);
  _qbank_add_usage($entity);
}

function qbank_dam_entity_delete($entity, $type) {
  list($entity_id) = entity_extract_ids($type, $entity);

  db_delete('file_usage')
    ->condition('module', 'media')
    ->condition('type', $type)
    ->condition('id', $entity_id)
    ->execute();
}
/**
 * Add file usage from file references in an entity's text fields.
 */
function _qbank_filter_add_file_usage_from_fields($entity_type, $entity) {
  // Track the total usage for files from all fields combined.
  $entity_files = qbank_entity_field_count_files($entity_type, $entity);

  list($entity_id, $entity_vid, $entity_bundle) = entity_extract_ids($entity_type, $entity);

  // When an entity has revisions and then is saved again NOT as new version the
  // previous revision of the entity has be loaded to get the last known good
  // count of files. The saved data is compared against the last version
  // so that a correct file count can be created for that (the current) version
  // id. This code may assume some things about entities that are only true for
  // node objects. This should be reviewed.
  // @TODO this conditional can probably be condensed
  if (empty($entity->revision) && empty($entity->old_vid) && empty($entity->is_new) && ! empty($entity->original)) {
    $old_files = qbank_entity_field_count_files($entity_type, $entity->original);
    foreach ($old_files as $fid => $old_file_count) {
      // Were there more files on the node just prior to saving?
      if (empty($entity_files[$fid])) {
        $entity_files[$fid] = 0;
      }
      if ($old_file_count > $entity_files[$fid]) {
        $deprecate = $old_file_count - $entity_files[$fid];
        // Now deprecate this usage
        $file = file_load($fid);
        if ($file) {
          file_usage_delete($file, 'media', $entity_type, $entity_id, $deprecate);
        }
        // Usage is deleted, nothing more to do with this file
        unset($entity_files[$fid]);
      }
      // There are the same number of files, nothing to do
      elseif ($entity_files[$fid] ==  $old_file_count) {
        unset($entity_files[$fid]);
      }
      // There are more files now, adjust the difference for the greater number.
      // file_usage incrementing will happen below.
      else {
        // We just need to adjust what the file count will account for the new
        // images that have been added since the increment process below will
        // just add these additional ones in
        $entity_files[$fid] = $entity_files[$fid] - $old_file_count;
      }
    }
  }

  // Each entity revision counts for file usage. If versions are not enabled
  // the file_usage table will have no entries for this because of the delete
  // query above.
  foreach ($entity_files as $fid => $entity_count) {
    if ($file = file_load($fid)) {
      file_usage_add($file, 'media', $entity_type, $entity_id, $entity_count);
    }
  }
}

/**
 * Parse file references from an entity's text fields and return them as an array.
 */
function qbank_filter_parse_from_fields($entity_type, $entity) {
  $file_references = array();

  foreach (qbank_filter_fields_with_text_filtering($entity_type, $entity) as $field_name) {
    if ($field_items = field_get_items($entity_type, $entity, $field_name)) {
      foreach ($field_items as $field_item) {
        if (!empty($field_item['value'])) {
          $doc = new DOMDocument();
          $doc->loadHTML($field_item['value']);
          $imgs = $doc->getElementsByTagName('img');

          foreach ($imgs as $img){
            $filename = explode('/',$img->getAttribute('src'));
            $filename = $filename[sizeof($filename)-1];
            $query = new EntityFieldQuery();
            $result = $query->entityCondition('entity_type', 'file')
              ->propertyCondition('uri', '%/' . $filename, 'LIKE')
              ->execute();
            $file_object = reset($result['file']);
            $file_references[] = array(
              'fid' => $file_object->fid,
              'filename' => $filename,
            );
          }
        }
      }
    }
  }

  return $file_references;
}

/**
 * Utility function to get the file count in this entity
 *
 * @param type $entity
 * @param type $entity_type
 * @return int
 */
function qbank_entity_field_count_files($entity_type, $entity) {
  $entity_files = array();
  foreach (qbank_filter_parse_from_fields($entity_type, $entity) as $file_reference) {
    if (empty($entity_files[$file_reference['fid']])) {
      $entity_files[$file_reference['fid']] = 1;
    }
    else {
      $entity_files[$file_reference['fid']]++;
    }
  }
  return $entity_files;
}

/**
 * Returns an array containing the names of all fields that perform text filtering.
 */
function qbank_filter_fields_with_text_filtering($entity_type, $entity) {
  list($entity_id, $revision_id, $bundle) = entity_extract_ids($entity_type, $entity);
  $fields = field_info_instances($entity_type, $bundle);

  // Get all of the fields on this entity that allow text filtering.
  $fields_with_text_filtering = array();
  foreach ($fields as $field_name => $field) {
    if (!empty($field['settings']['text_processing'])) {
      $fields_with_text_filtering[] = $field_name;
    }
  }

  return $fields_with_text_filtering;
}

function _qbank_add_usage($node){
  $query = db_select('file_usage', 'fu');
  $query->join('file_managed', 'fm', 'fu.fid = fm.fid');
  $query->fields('fm', array('fid', 'uri','qbank_origin_id', 'uid'));
  $query->fields('fu', array('id'));
  $query->condition('fu.id', $node->nid, '=');
  $query->condition('fm.qbank_origin_type', 'qbank', '=');
  $r = $query->execute();

  $qbankapi = _get_qbankapi();

  $qbank_session = $qbankapi->events()->session(
    variable_get('qbank_session_id', 0),
    variable_get('qbank_deployment_site', 0),
    gethostbyname($_SERVER['HTTP_HOST']),
    'drupal'
  );

  while($record = $r->fetchAssoc()) {
    $user = user_load($record['uid']);

    $mediaUsage = new MediaUsage([
      'mediaId' => $record['qbank_origin_id'],
      'mediaUrl' => file_create_url($record['uri']),
      'pageUrl' => url('node/' . $record['id'], array('absolute' => TRUE)),
      'language' => 'eng', //Three character language ISO
      'context' => [
        'localID' => $record['fid'],
        'cropCoords' => [],
        'pageTitle' => $node->title,
        'createdByName' => $user->name,
        'createdByEmail' => $user->mail
      ]
    ]);

    $mediaUsageResponse = $qbankapi->events()->addUsage(
      $qbank_session,
      $mediaUsage
    );
  }
}