<?php
/**
 * Get QBank 3 API wrapper for PHP
 * https://github.com/QBNK/qbank3api-phpwrapper
 * Install via composer or download the package and install manually
 */

/**
 * Default uses when wrapper is installed through composer, may need to be updated if installed manually
 */
use Doctrine\Common\Cache\PhpFileCache;
use QBNK\QBank\API\CachePolicy;
use QBNK\QBank\API\Credentials;
use QBNK\QBank\API\Model\MediaUsage;
use QBNK\QBank\API\QBankApi;

/**
 * Require an autoloader, if you're not using one already, composer will include one under vendor/autoload.php
 */
require_once('/path/to/autoload.php');


/**
 * Client specific QBank credentials. The API implements Oauth2 authentication. The user/pass represents normal
 * login credentials to the respective QBank. Note that the API-users are sometimes restricted from logging in
 * to the backend
 */
define('HOST', 'xxxx.qbank.se');
define('CLIENT_ID', 'xxxx');
define('USERNAME', 'xxxx');
define('PASSWORD', 'xxxx');

/**
 * The deployment site ID controls which content is displayed in the connector. It is possible to run the
 * connector without a deployment site ID but it is not recommended as all thumbnails and previews will be fetched
 * through the API instead of read from already converted sources.
 * The deployment site ID is also different from each client and needs to be dynamically configurable like the
 * credentials above.
 */
define('DEPLOYMENT_SITE_ID', 0);

try {
	/**
	 * Setup QBank API
	 */
	$QBankAPI = new QBankApi(
		HOST,
		new Credentials(
			CLIENT_ID,
			USERNAME,
			PASSWORD
		),
		array(
			'log' => false,
			/**
			 * Cache is always recommended to be globally set, and can be fine-tuned by individual API calls as needed
			 */
			'cache' => new PhpFileCache('cache', 'appcache'),
			'cachePolicy' => new CachePolicy(false, 3600)
		)
	);

	/**
	 * The javascript connector requires that the backend supplies the access_token.
	 * The API won't actually get a valid token before an API call is made,
	 * any call will do, getting the current API user for example
	 */
	$me = $QBankAPI->accounts()->retrieveCurrentUser();

	define('ACCESS_TOKEN', $QBankAPI->getToken()->accessToken);

} catch (Exception $e) {
	var_dump($e);
}
?>

<html>
	<head>
		<!-- Get the connector javascript from the respective QBank you're connecting to
		As long as you include the file directly from us, you'll recieve updates automatically.
		If you download the script and host locally, you will need to update it manually! -->
		<script src="<?= HOST ?>/connector/qbank-connector.min.js"></script>
	</head>
	<body>
		<!-- You get to decide where to embed the connector, the connector will then create an <iframe> in the given
			wrapper and load itself in there. So you create the dialog/window/whatever as well as width/height to be used -->
		<div id="wrapper" style="height: 500px;"></div>


		<script>
			/**
			 * Initialize the QBank object which can then be used to create any number of connector dialogs.
			 */
			var QBC = new QBankConnector({
				deploymentSite: <?= DEPLOYMENT_SITE_ID ?>,
				api: {
					version: 'v1',
					host: '<?= HOST ?>',
					access_token: '<?= ACCESS_TOKEN ?>',
					protocol: 'https'
				},
				gui: {
					basehref: '<?= HOST ?>/connector/'
				}
			});

			/**
			 * Open up a qbank connector! The only required parameter is "container".
			 * For examples on how the different parameters can be used, visit http://connector.qbank.se/test
			 */
			var qbankMediaPicker = new QBC.mediaPicker({
				container: '#wrapper',
				onSelect: function(selectedMedia, selectedFiles, usingPreviousUsage) {
					/**
					 * selectedMedia contains the media data object
					 * selectedFiles contains specific data about how to fetch the actual file of that media that was chosen
					 * usingPreviousUsage contains information about a which previous usage to use, if a previous usage was chosen
					 */

					/**
					 * It is up to you and your configuration to insert the file(s) that were chosen into your integration
					 * You probably want to take the URL's you find in "selectedFiles", send to some endpoint where they
					 * are saved to the local site and given some local ID, and then insert the local path into whatever it is
					 * the integration needs it for!
					 */

					/**
					 * We strongly encourage you to report back to QBank that this media has been used in something.
					 * This is an example of how that can be done. Depending on the integration, it may not be optimal to
					 * report the usage right away, but instead to use some sort of job that looks through the local files
					 * and reports uses. A CMS for example that publishes pages shouldn't report the usage as soon as the
					 * image is inserted into the post, since the page may be deleted right away and not even saved as a
					 * draft. The CMS will in that situation need to add some hooks to publish events, or run a job that
					 * looks at published pages and finds QBank files being used there.
					 */
					if (!usingPreviousUsage) {
						/**
						 * Reporting the usage right away through XHR could look something like this
						 * See the bottom of this example for how to register the usage
						 */
						var xhr = new XMLHttpRequest(),
							data = new FormData();
						data.append('media', JSON.stringify(selectedMedia));
						data.append('files', JSON.stringify(selectedFiles));

						xhr.open("POST", "example.php");
						xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
						xhr.send(data);
					}
				},
				onReady: function() {},
				onClose: function() {},
				title: "QBank Connector",
				mediaId: null,
				iframeid: 'qbank-connector-iframe-' + new Date().getTime(),
				modules: {
					folders: true,
					categories: true,
					moodboards: true,
					settings: true,
					content: {
						header: true,
						toolbar: true,
						details: true
					},
					upload: {
						maxFileSize: 0,
						chunkSize: "10mb"
					},
					imageTool: {
						crop: true
					},
					detail: {
						showUseButton: true
					},
					searchResult: {
						showUseButton: false
					}
				},
				requestedTemplates: null,
				defaultUseSize: '600x600',
				defaultUseExtension: 'jpg'
			});
		</script>
	</body>
</html>

<?php

/**
 * Check if we need to report a usage!
 */
if (!empty($_POST['media'])) {
	@session_start();

	/**
	 * When reporting a usage, we need to report from which source the usage came. One CLIENT_ID could be used to communicate
	 * usages from many different sources. This paramater is also dynamic and client specific
	 */
	define('SESSION_SOURCE_ID', 0);

	/**
	 * We need to get a session ID. The session ID is used to report any events that transpire during this API session.
	 * It also gives QBank some information about who/what/where is doing the reporting
	 */
	if (!$_SESSION['qbankApiSessionId']) {
		$_SESSION['qbankApiSessionId'] = $QBankAPI->events()->session(
			SESSION_SOURCE_ID,
			session_id(),
			$_SERVER['REMOTE_ADDR'],
			$_SERVER['HTTP_USER_AGENT']
		);
	}

	try {
		$media = json_decode($_POST['media'], true);
		foreach (json_decode($_POST['files'], true) as $file) {
			$mediaUsage = new MediaUsage([
				'mediaId' => $media['mediaId'],
				'mediaUrl' => 'https://your-site.com/public/path/to/the/file.ext',
				'pageUrl' => 'https://your-site.com/page-where-file-is-visible.html',
				'language' => 'eng', //Three character language ISO
				/**
				 * Everything you put in the "context" array will be available when you fetch usages from the API.
				 * This is useful for communicating local ID's you have saved to each file, so you could read usages
				 * for a media in QBank and get references needed by your application to for example find all local
				 * files that have been used.
				 *
				 * cropCoords, pageTitle, createdByName and createdByEmail should be filled out if possible.
				 */
				'context' => [
					'localID' => 'if-you-have-a-local-id-to-this-file',
					'cropCoords' => (!empty($file['crop']) ? $file['crop'] : []),
					'pageTitle' => 'Title of the page where the file is visible',
					'createdByName' => 'xxxx xxxx',
					'createdByEmail' => 'xxxx@xxxx.xx'
				]
			]);

			/**
			 * Post the usage to QBank!
			 * You will get a MediaUsage object back which contains the ID QBank has registered for this usage.
			 * You will need that ID when you want to report that a file is no longer being used. We suggest that you
			 * save the MediaUsage ID together with your local file ID.
			 */
			$mediaUsage = $QBankAPI->events()->addUsage(
				$_SESSION['qbankApiSessionId'],
				$mediaUsage
			);
		}
	} catch (Exception $e) {
		var_dump($e);
	}
}